@@ -36,7 +36,10 @@ pub enum CryptoError {
     NotDeserializableToBaseDataType,
 
     /// Wrong nonce was provided during seal/unseal operation
-    WrongNonceType
+    WrongNonceType,
+
+    /// A signature was not verifiable
+    BadSignature,
 }
 
 impl Error for CryptoError {
@@ -49,7 +52,8 @@ impl Error for CryptoError {
             CryptoError::InvalidSeedLength { .. } => None,
             CryptoError::NotDowncastable => None,
             CryptoError::NotDeserializableToBaseDataType => None,
-            CryptoError::WrongNonceType => None
+            CryptoError::WrongNonceType => None,
+            CryptoError::BadSignature => None
         }
     }
 }
@@ -98,6 +102,9 @@ impl Display for CryptoError {
             CryptoError::WrongNonceType => {
                 write!(f, "Invalid type of nonce was provided for the operation")
             }
+            CryptoError::BadSignature => {
+                write!(f, "Signature verification failed")
+            }
         }
     }
 }
